@@ -11,6 +11,8 @@
 
 #include <knotification.h>
 
+#include <memory>
+
 class KNotification;
 class QPixmap;
 class KNotificationPlugin;
@@ -53,6 +55,7 @@ public:
 
 private Q_SLOTS:
     void notificationClosed();
+    void xdgActivationTokenReceived(int id, const QString &token);
     void notificationActivated(int id, int action);
     void notificationReplied(int id, const QString &text);
     void notifyPluginFinished(KNotification *notification);
@@ -60,7 +63,7 @@ private Q_SLOTS:
 
 private:
     struct Private;
-    Private *const d;
+    std::unique_ptr<Private> const d;
     KNotificationManager();
 
     friend class KNotificationManagerSingleton;
