@@ -5,13 +5,14 @@
 */
 
 #include "notifybysnore.h"
+#include "debug_p.h"
 #include "knotification.h"
 #include "knotifyconfig.h"
-#include "debug_p.h"
+#include "knotificationreplyaction.h"
 
+#include <QGuiApplication>
 #include <QIcon>
 #include <QLocalSocket>
-#include <QGuiApplication>
 
 #include <snoretoastactions.h>
 
@@ -44,30 +45,36 @@
  * appID: use as-is from your app's QCoreApplication::applicationName() when installing the shortcut.
  * NOTE: Install the shortcut in Windows Start Menu folder.
  * For example, check out Craft Blueprint for Quassel-IRC or KDE Connect.
-*/
+ */
 
-namespace {
-    const QString SnoreToastExecName() { return QStringLiteral("SnoreToast.exe"); }
+namespace
+{
+const QString SnoreToastExecName()
+{
+    return QStringLiteral("SnoreToast.exe");
+}
 }
 
-NotifyBySnore::NotifyBySnore(QObject* parent) :
-    KNotificationPlugin(parent)
+NotifyBySnore::NotifyBySnore(QObject *parent)
+    : KNotificationPlugin(parent)
 {
     m_server.listen(QString::number(qHash(qApp->applicationDirPath())));
     connect(&m_server, &QLocalServer::newConnection, this, [this]() {
-        QLocalSocket* responseSocket = m_server.nextPendingConnection();
-        connect(responseSocket, &QLocalSocket::readyRead, [this, responseSocket](){
+        QLocalSocket *responseSocket = m_server.nextPendingConnection();
+        connect(responseSocket, &QLocalSocket::readyRead, [this, responseSocket]() {
             const QByteArray rawNotificationResponse = responseSocket->readAll();
             responseSocket->deleteLater();
 
-            const QString notificationResponse =
-                        QString::fromWCharArray(reinterpret_cast<const wchar_t *>(rawNotificationResponse.constData()),
-                                            rawNotificationResponse.size() / sizeof(wchar_t));
-            QMap<QString, QStringRef> notificationResponseMap;
-            for (auto &str : notificationResponse.splitRef(QLatin1Char(';'))) {
+            const QString notificationResponse = QString::fromWCharArray(reinterpret_cast<const wchar_t *>(rawNotificationResponse.constData()),
+                                                                         rawNotificationResponse.size() / sizeof(wchar_t));
+            qCDebug(LOG_KNOTIFICATIONS) << notificationResponse;
+
+            QMap<QString, QStringView> notificationResponseMap;
+            for (const auto str : QStringView(notificationResponse).split(QLatin1Char(';'))) {
                 const int equalIndex = str.indexOf(QLatin1Char('='));
                 notificationResponseMap.insert(str.mid(0, equalIndex).toString(), str.mid(equalIndex + 1));
             }
+
             const QString responseAction = notificationResponseMap[QStringLiteral("action")].toString();
             const int responseNotificationId = notificationResponseMap[QStringLiteral("notificationId")].toInt();
 
@@ -77,8 +84,7 @@ NotifyBySnore::NotifyBySnore(QObject* parent) :
             const auto iter = m_notifications.constFind(responseNotificationId);
             if (iter != m_notifications.constEnd()) {
                 notification = iter.value();
-            }
-            else {
+            } else {
                 qCWarning(LOG_KNOTIFICATIONS) << "Received a response for an unknown notification.";
                 return;
             }
@@ -107,14 +113,18 @@ NotifyBySnore::NotifyBySnore(QObject* parent) :
                 qCDebug(LOG_KNOTIFICATIONS) << "User clicked an action button in the toast.";
                 const QString responseButton = notificationResponseMap[QStringLiteral("button")].toString();
                 QStringList s = m_notifications.value(responseNotificationId)->actions();
-                int actionNum = s.indexOf(responseButton) + 1;       // QStringList starts with index 0 but not actions
+                int actionNum = s.indexOf(responseButton) + 1; // QStringList starts with index 0 but not actions
                 Q_EMIT actionInvoked(responseNotificationId, actionNum);
                 break;
             }
 
-            case SnoreToastActions::Actions::TextEntered:
-                qCWarning(LOG_KNOTIFICATIONS) << "User entered some text in the toast. This is is not handled yet.";
+            case SnoreToastActions::Actions::TextEntered: {
+                qCWarning(LOG_KNOTIFICATIONS) << "User entered some text in the toast.";
+                const QString replyText = notificationResponseMap[QStringLiteral("text")].toString();
+                qCWarning(LOG_KNOTIFICATIONS) << "Text entered was :: " << replyText;
+                Q_EMIT replied(responseNotificationId, replyText);
                 break;
+            }
 
             default:
                 qCWarning(LOG_KNOTIFICATIONS) << "Unexpected behaviour with the toast. Please file a bug report / feature request.";
@@ -139,64 +149,71 @@ void NotifyBySnore::notify(KNotification *notification, KNotifyConfig *config)
     Q_UNUSED(config);
     // HACK work around that notification->id() is only populated after returning from here
     // note that config will be invalid at that point, so we can't pass that along
-    QMetaObject::invokeMethod(this, [this, notification](){ NotifyBySnore::notifyDeferred(notification); }, Qt::QueuedConnection);
+    QMetaObject::invokeMethod(
+        this,
+        [this, notification]() {
+            NotifyBySnore::notifyDeferred(notification);
+        },
+        Qt::QueuedConnection);
 }
 
-void NotifyBySnore::notifyDeferred(KNotification* notification)
+void NotifyBySnore::notifyDeferred(KNotification *notification)
 {
     m_notifications.insert(notification->id(), notification);
 
-    const QString notificationTitle = ((!notification->title().isEmpty()) ? notification->title()
-                                                                          : qApp->applicationDisplayName());
-    QStringList snoretoastArgsList {
-        QStringLiteral("-id"), QString::number(notification->id()),
-        QStringLiteral("-t"), notificationTitle,
-        QStringLiteral("-m"), stripRichText(notification->text()),
-        QStringLiteral("-appID"), qApp->applicationName(),
-        QStringLiteral("-pid"), QString::number(qApp->applicationPid()),
-        QStringLiteral("-pipename"), m_server.fullServerName()
-    };
+    const QString notificationTitle = ((!notification->title().isEmpty()) ? notification->title() : qApp->applicationDisplayName());
+    QStringList snoretoastArgsList{QStringLiteral("-id"),
+                                   QString::number(notification->id()),
+                                   QStringLiteral("-t"),
+                                   notificationTitle,
+                                   QStringLiteral("-m"),
+                                   stripRichText(notification->text()),
+                                   QStringLiteral("-appID"),
+                                   qApp->applicationName(),
+                                   QStringLiteral("-pid"),
+                                   QString::number(qApp->applicationPid()),
+                                   QStringLiteral("-pipename"),
+                                   m_server.fullServerName()};
 
     // handle the icon for toast notification
     const QString iconPath = m_iconDir.path() + QLatin1Char('/') + QString::number(notification->id());
-    const bool hasIcon = (notification->pixmap().isNull()) ? qApp->windowIcon().pixmap(1024, 1024).save(iconPath, "PNG")
+    const bool hasIcon = (notification->pixmap().isNull()) ? qApp->windowIcon().pixmap(1024, 1024).save(iconPath, "PNG") //
                                                            : notification->pixmap().save(iconPath, "PNG");
     if (hasIcon) {
         snoretoastArgsList << QStringLiteral("-p") << iconPath;
     }
 
-    // add actions if any
-    if (!notification->actions().isEmpty()) {
+    // if'd below, because SnoreToast currently doesn't support both textbox and buttons in the same notification
+    if (notification->replyAction()) {
+        snoretoastArgsList << QStringLiteral("-tb");
+    } else if (!notification->actions().isEmpty()) {
+        // add actions if any
         snoretoastArgsList << QStringLiteral("-b") << notification->actions().join(QLatin1Char(';'));
     }
 
-    QProcess* snoretoastProcess = new QProcess();
-    connect(snoretoastProcess, &QProcess::readyReadStandardError,
-            [snoretoastProcess, snoretoastArgsList]() {
-                const auto data = snoretoastProcess->readAllStandardError();
-                qCDebug(LOG_KNOTIFICATIONS) << "SnoreToast process stderr:"
-                    << snoretoastArgsList << data;
+    QProcess *snoretoastProcess = new QProcess();
+    connect(snoretoastProcess, &QProcess::readyReadStandardError, [snoretoastProcess, snoretoastArgsList]() {
+        const auto data = snoretoastProcess->readAllStandardError();
+        qCDebug(LOG_KNOTIFICATIONS) << "SnoreToast process stderr:" << snoretoastArgsList << data;
     });
-    connect(snoretoastProcess, &QProcess::readyReadStandardOutput,
-            [snoretoastProcess, snoretoastArgsList]() {
-                const auto data = snoretoastProcess->readAllStandardOutput();
-                qCDebug(LOG_KNOTIFICATIONS) << "SnoreToast process stdout:"
-                    << snoretoastArgsList << data;
+    connect(snoretoastProcess, &QProcess::readyReadStandardOutput, [snoretoastProcess, snoretoastArgsList]() {
+        const auto data = snoretoastProcess->readAllStandardOutput();
+        qCDebug(LOG_KNOTIFICATIONS) << "SnoreToast process stdout:" << snoretoastArgsList << data;
     });
-    connect(snoretoastProcess, &QProcess::errorOccurred, this,
-            [this, snoretoastProcess, snoretoastArgsList, iconPath](QProcess::ProcessError error) {
-                qCWarning(LOG_KNOTIFICATIONS) << "SnoreToast process errored:"
-                    << snoretoastArgsList << error;
-                snoretoastProcess->deleteLater();
-                QFile::remove(iconPath);
+    connect(snoretoastProcess, &QProcess::errorOccurred, this, [this, snoretoastProcess, snoretoastArgsList, iconPath](QProcess::ProcessError error) {
+        qCWarning(LOG_KNOTIFICATIONS) << "SnoreToast process errored:" << snoretoastArgsList << error;
+        snoretoastProcess->deleteLater();
+        QFile::remove(iconPath);
     });
-    connect(snoretoastProcess, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished), this,
+    connect(snoretoastProcess,
+            qOverload<int, QProcess::ExitStatus>(&QProcess::finished),
+            this,
             [this, snoretoastProcess, snoretoastArgsList, iconPath](int exitCode, QProcess::ExitStatus exitStatus) {
                 qCDebug(LOG_KNOTIFICATIONS) << "SnoreToast process finished:" << snoretoastArgsList;
                 qCDebug(LOG_KNOTIFICATIONS) << "code:" << exitCode << "status:" << exitStatus;
                 snoretoastProcess->deleteLater();
                 QFile::remove(iconPath);
-    });
+            });
 
     qCDebug(LOG_KNOTIFICATIONS) << "SnoreToast process starting:" << snoretoastArgsList;
     snoretoastProcess->start(SnoreToastExecName(), snoretoastArgsList);
@@ -212,11 +229,7 @@ void NotifyBySnore::close(KNotification *notification)
 
     m_notifications.remove(notification->id());
 
-    const QStringList snoretoastArgsList{ QStringLiteral("-close"),
-                                          QString::number(notification->id()),
-                                          QStringLiteral("-appID"),
-                                          qApp->applicationName()
-                                        };
+    const QStringList snoretoastArgsList{QStringLiteral("-close"), QString::number(notification->id()), QStringLiteral("-appID"), qApp->applicationName()};
 
     qCDebug(LOG_KNOTIFICATIONS) << "Closing notification; SnoreToast process arguments:" << snoretoastArgsList;
     QProcess::startDetached(SnoreToastExecName(), snoretoastArgsList);
@@ -230,4 +243,3 @@ void NotifyBySnore::update(KNotification *notification, KNotifyConfig *config)
     Q_UNUSED(config);
     qCWarning(LOG_KNOTIFICATIONS) << "updating a notification is not supported yet.";
 }
-
